%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% THIS PROGRAM COMPUTES THE THEORETICAL MOMENTS AND THE VECTOR OF DISTANCES  
% BETWEEN THEORETICAL AND EMPIRICAL MOMENTS TO BE MINIMIZED.

% MODEL SPECIFICATION: STATIONARY FULL MODEL.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function f = estimation_fct_unrestr(x,cov_mat,exper_mat,lag_mat,diagonal_mat,sigma_ab_var,sigma_b_var)



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STEP A: EXTRACT CURRENT PARAMETER VALUES AND COMPUTE AUXILIARY PARAMETERS
%         NEEDED IN COMPUTATION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

sigma_alpha = x(1);
sigma_ab = x(2);
sigma_beta = x(3);
rho = x(4);
sigma_ARinit = x(5);
sigma_u = x(6);
sigma_eps = x(7);

g0_coeff = x(8);
g1_coeff = x(9);
g2_coeff = x(10);
g3_coeff = x(11);
g4_coeff = x(12);




% Prepare a number of variables from the inputs:
slope_het_coeff = [sigma_ab; sigma_beta];
slope_het_vars = [sigma_ab_var sigma_b_var];                            % NOTE: No scaling!

covar_impute = cov_mat;
exper = exper_mat;
lag = lag_mat;
diagonal = diagonal_mat;







%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STEP B: COMPUTE THE AR(1)-PROCESS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% The AR(1)-process is given by z(ibt) = rho*z(ib,t-1) + v(ibt)
% with var(v(ibt)) = sigma_v^2 and initial condition var(z(ib0)) =
% sigma_v0^2. Consequently, the variances follow the process
% V(ibt) = (rho^2)*V(ib,t-1) + sigma_v^2 
% with initial condition V(ib0) = sigma_v0^2.
% Note that the processes are stationary. As a consequence, the relevant
% variance for an individual is uniquely identified by (experience). I thus 
% loop over this variable below.


% Prepare Matrix to be filled with the heteroscedastic AR(1):
AR_matrix = zeros(max(exper)+1,1);


% Initialize the Process:
update = sigma_ARinit;
AR_matrix(1,1) = update;

for k = 2:(max(exper)+1)       

    exper_help = k-1;                                           %experience starts at zero

    var_z = g0_coeff + g1_coeff*exper_help + g2_coeff*(exper_help^2) + ...
        + g3_coeff*(exper_help^3) + g4_coeff*(exper_help^4);

    var_AR = (rho^2)*update + var_z;
    AR_matrix(k,1) = var_AR;
    update = var_AR;

end







%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STEP C: MATCH THE AR_matrix TO THE EMPIRICAL AUTO-COVARIANCES
%         NOTE: The match is by experience!
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

exper_help = exper + 1;
AR_var = AR_matrix(exper_help);
AR_var = (rho.^lag).*AR_var;






%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STEP D: COMPUTE THE PERMANENT COMPONENT:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

perm_component = sigma_alpha + slope_het_vars*slope_het_coeff + sigma_u*exper;





%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STEP E: COMPUTE THE THEORETICAL MOMENTS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

covar_hat = perm_component + AR_var + sigma_eps*diagonal;
      




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STEP F: COMPUTE THE SQUARED DEVIATION BETWEEN THEORETICAL AND EMPIRICAL
%         MOMENTS:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

f = covar_impute - covar_hat;









