%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MONTE CARLO ANALYSIS
%
% This program carries out a Monte-Carlo analysis. First, I set parameters
% and variables that are held constant across simulations. Second, I loop 
% over simulation draws of entire histories. Within that loop, the 
% covariance structure is calculated and the estimation is carried out.
% Estimates are saved in a vector. It is best to plot them in Stata.

% NOTE: This program simulates the covariance structure when the model
%       is the benchmark specification, but without time effects. The
%       covariance structure is thus aggregated across cohorts.
%       Importantly, from each sample (and thus for each covariance
%       structure) I estimate three different specifications:
%       (1) No restrictions on sigma(beta) and sigma(init)
%       (2) Impose sigma(beta) == 0
%       (3) Impose sigma(init) == 0

%       Exercise (1) allows for studying whether parameters can be
%       estimated without bias. Exercises (2) and (3) explore the bias from
%       excluding a model component.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear
matlabpool open 8


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STEP 1: SET/EXTRACT PARAMETERS

% NOTE: This part of the program is the same for all 4 model variations
%       explored in the Monte Carlo. Model-specific restrictions are
%       imposed in STEP 2.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% SET PARAMETERS DESCRIBING SIMULATION SIZES
N_mc = 1000;                                %Number of Monte-Carlo repetitions
max_exper = 108;                            %Simulate 108 quarters.
seed_init = 13654;


% LOAD THE SAMPLE SIZES BY EXPERIENCE (IN YEARS); WITHIN YEARS, SAMPLE SIZE
% IS ASSUMED TO BE EQUALLY SPLIT ACROSS QUARTERS. THESE VALUES ARE THE
% ACTUAL ONES DOCUMENTED IN THE APPENDIX TABLE OF THE PAPER.
load 'C:\Users\FH\Documents\MATLAB\Covariance Structures\High School Degree Sample\non-stationary models\R&R Quantitative Economics\Monte Carlo\sample_size.mat'
N_ind = ceil(sample_size(1)/4);                     %the sample sizes are given on the annual level.


% SET THE VALUE OF PARAMETERS:
% NOTE: For all parameters other than the time effects and the HIP 
%       component, use the estimates from the stationary version. If HIP is
%       assumed to be present, use the estimates from the Hryshko
%       specification.
% NOTE2: Depending on the model specification considered, additional
%       restrictions on the parameters are imposed. This is done in STEP2.
sigma_alpha = 0.053;
sigma_ab = -0.005/10;
sigma_beta = 0.005/1000;
rho = 0.757;
sigma_ARinit = 0.136;
sigma_u = 0.011/10;
sigma_eps = 0.003;

g0 = 0.026;
g1 = -0.001;
g2 = 2.98/(10^5);
g3 = -3.67/(10^7);
g4 = 1.64/(10^9);



% SET THE INITIAL CONDITIONS FOR PARAMETERS:
x0_init = zeros(12,1);
x0_init(1) = 0.03;                       %approx the avg. long-run value of auto-covariances
x0_init(4) = 0.95;                       %arbitrary
x0_init(5) = 0.05;                       %approx. the avg. difference between initial and long-run value of autocovariances
x0_init(6) = 0.001;                      %approx. slope of linear part of life-cycle variance profiles
x0_init(8) = 0.05;                       %same value as initial condition.                          





%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STEP 2: MONTE CARLO ANALYSIS.

% The steps are:
% - impose additional restrictions on parameters, depending on model
%   specification.
% - start loop over Monte Carlo draws.
% - Within the loop: (i) compute earnings histories; (ii) compute
%   covariance structure, thereby imposing random attrition so that sample
%   sizes are the ones in the actual data; (iii) estimate the parameters.
% - After the loop: calculate distributional characteristics of the
%   parameter estimates. 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Calculate variables that are stable across iterations:
exper_vec = (0:(max_exper-1))';
exper_rep = repmat(exper_vec,[1 N_ind]);


% Compute the number of individuals entering into the calculation of the
% covariances at each level of experience. This controls for random
% attrition. The loop below is messy because we need to translate sample
% sizes into quarterly rather than annual data:
N_exper = zeros(max_exper,1);

for i = 1:max_exper
    
    j = i/4;
    j_ceil = ceil(j);
    N_exper(i) = ceil(sample_size(j_ceil)/4);
    
    % The following commands make sure that within year, the quarterly
    % numbers add up to the corresponding annual number
    if j == j_ceil
        N_exper(i) = sample_size(j) - 3*N_exper(i-1);
    end
            
end

clear i j j_ceil



% Set the options for the optimization:
options_help = optimset('lsqnonlin');
options = optimset(options_help,'display','off','LargeScale','off','LevenbergMarquardt','off','MaxFunEvals',30000,'MaxIter',1000);



% Initialize matrix of parameter estimates to be filled:
parms_unrestr = zeros(length(x0_init),N_mc);
parms_noHIP = zeros(length(x0_init)-2,N_mc);
parms_noInit = zeros(length(x0_init)-1,N_mc);



% Start simulation:
parfor i = 1:N_mc
    
    N_exper_help = N_exper;                     %helpful because of parfor loop
    
    % Get random draws. Note that draws are taken within the loop since
    % otherwise one runs into memory problems if N_mc is large. The seed
    % can be controlled by setting it to (seed_init+i).
    % We need random draws for the initial condition, incl. the persistent
    % one, the permanent shocks, the persistent shocks, and measurement
    % error. For each individual there are thus [2 + 2*max_exper +
    % (max_exper - 1)] draws. The first term is for intercept- and slope
    % heterogeneity; the second term is for AR(1) (incl. initial condition)
    % and measurement error; the third term is for the unit roots, which
    % has degenerate initial condition.
    seed_i = seed_init + i;
    N_draws = 2 + 2*max_exper + (max_exper - 1);
    s = RandStream('mt19937ar','Seed',seed_i);
    draws_i = randn(s,[N_draws,N_ind]);
    
    
    % Simulate the permanent component:
    corr_ab = sigma_ab/sqrt((sigma_alpha*sigma_beta));
    corr_ab = max(corr_ab,-1);
    corr_ab = min(corr_ab,1);

    draws_het = draws_i(1:2,:);
    alpha = sqrt(sigma_alpha)*draws_het(1,:);
    beta = sigma_ab*alpha/sigma_alpha + sqrt(sigma_beta)*sqrt(max(1-corr_ab^2,0))*draws_het(2,:);
    
    
    
    % Simulate AR(1), starting from the initial condition:
    draws_ar = draws_i(3:(3+max_exper-1),:);                                    %one set of draws for each exper (incl zero)
    
    sigma_AR = g0+g1*exper_vec + g2 *exper_vec.^2 + g3 *exper_vec.^3 + g4 *exper_vec.^4;        %heteroscedasticity
    sigma_AR(1) = sigma_ARinit;                                                 %initial condition (exper = 0)
    sigma_AR_std = sqrt(sigma_AR);
    
    sigma_AR_std = repmat(sigma_AR_std,[1 N_ind]);
    draws_ar = sigma_AR_std.*draws_ar; 
    
    Z = zeros(max_exper,N_ind);
    Z(1,:) = draws_ar(1,:); 
    
    for t = 2:max_exper
        draws_help = draws_ar(t,:);
        Z(t,:)= rho*Z(t-1,:) + draws_help;
    end   
    
    
    
    % Simulate unit roots, which is assumed to be homoscedastic with a
    % zero initial condition:
    draws_ur = draws_i(3+max_exper:(3+2*max_exper-2),:);                %"-2" because exper == 0 needs to be treated as non-random (zero initial condition)  
    draws_ur = sqrt(sigma_u)*draws_ur;
    draws_ur = [zeros(1,N_ind); draws_ur];
    U = cumsum(draws_ur);
   
    
    
    % Draw measurement error:
    draws_trans = draws_i((3+2*max_exper-1):N_draws,:); 
    E = sqrt(sigma_eps)*draws_trans;
    
    
    
    % Add everything up to compute residual earnings:
    alpha = repmat(alpha,[max_exper 1]);
    beta = repmat(beta,[max_exper 1]);    
    earn = (alpha + beta.*exper_rep + U) + Z + E;
    
    
    
    % Compute covariance matrix, looping over experience and the lag; the
    % vectorized covariance matrix will have exper_max*(exper_max+1)/2
    % unique elements. Note that in the actual data, the lag is actually
    % the lead. Start with an initialization:
    cov_mat = zeros(max_exper*(max_exper+1)/2,1);
    exper_mat = cov_mat;
    lag_mat = cov_mat;
    
    row_index = 1;
   
    for t = 1:max_exper
        for k = 0:(max_exper-t)
            
            % Extract the two set of earnings data and calculate the 
            % covariance between them. Need to take attrition into account.
            earn_t = earn(t,:);
            earn_k = earn(t+k,:);
            earn_tk = [earn_t' earn_k'];
            
            N_attr = N_exper_help(t+k);                         %need to choose the N for row with smallest number of elements. This is decreasing in exper.
            
            if N_attr < N_ind
                earn_tk((N_attr+1):N_ind,:) = [];
            else
            end
            
            cov_help = cov(earn_tk);
            
            % Record as data, together with the variables identifying them:
            cov_mat(row_index) = cov_help(1,2);
            exper_mat(row_index) = t-1;                          %exper starts at zero in the model!
            lag_mat(row_index) = k;
            
            row_index = row_index + 1;
                       
        end
    end
            
    diagonal_mat = lag_mat == 0;
    sigma_ab_var = 2*exper_mat + lag_mat;
    sigma_b_var = exper_mat.*(exper_mat + lag_mat);
    
    
    
    % NOW ESTIMATE THE PARAMETERS IMPOSING THREE DIFFERENT RESTRICTIONS:
    % (1) NONE
    % (2) NO HIP
    % (3) NO INITIAL PERSISTENT SHOCK.
    
    % Start without imposing any restrictions:
    x0 = x0_init;
    f1 = @(x)estimation_fct_unrestr(x,cov_mat,exper_mat,lag_mat,diagonal_mat,sigma_ab_var,sigma_b_var);
    x = lsqnonlin(f1,x0,[],[],options);
    parms_unrestr(:,i) = x;
 
    
    % Omit HIP (erroneously):   
    x0 = x0_init;
    x0(2:3) = [];    
    f2 = @(x)estimation_fct_noHIP(x,cov_mat,exper_mat,lag_mat,diagonal_mat,sigma_ab_var,sigma_b_var);
    x = lsqnonlin(f2,x0,[],[],options);
    parms_noHIP(:,i) = x;    
 
    
    % Omit sigma_init (erroneously):   
    x0 = x0_init;
    x0(5) = [];    
    f3 = @(x)estimation_fct_noInit(x,cov_mat,exper_mat,lag_mat,diagonal_mat,sigma_ab_var,sigma_b_var);
    x = lsqnonlin(f3,x0,[],[],options);
    parms_noInit(:,i) = x;    
        
    
    i
    
end


save('parms_unrestr','parms_unrestr')
save('parms_noHIP','parms_noHIP')
save('parms_noInit','parms_noInit')



% CALCULATE BIAS AND VARIANCE:
% Unrestricted Model:
parm_true = [sigma_alpha sigma_ab sigma_beta rho sigma_ARinit sigma_u sigma_eps g0 g1 g2 g3 g4];
std_unrestr = std(parms_unrestr');
bias_unrestr = mean(parms_unrestr') - parm_true;

% No HIP:
parm_true_noHIP = parm_true;
parm_true_noHIP(2:3) = [];
std_noHIP = std(parms_noHIP');
bias_noHIP = mean(parms_noHIP') - parm_true_noHIP;

% No Init:
parm_true_noInit = parm_true;
parm_true_noInit(5) = [];
std_noInit = std(parms_noInit');
bias_noInit = mean(parms_noInit') - parm_true_noInit;


matlabpool close


